@@ -122,11 +122,15 @@ cluswilcox.test.ranksum.rgl.clus <- function(x, cluster, group,
             summary.mat[which(Csize == i & Str == j), "mgv"] <- length(temp.grp[temp.grp == 1])
             summary.mat[which(Csize == i & Str == j), "Ngv"] <- length(temp.clus)
             summary.mat[which(Csize == i & Str == j), "Rsumgv"] <- sum(temp.rksum)
-            summary.mat[which(Csize == i & Str == j), "VRgv"] <- var(temp.rksum) * (length(temp.clus) - 1)
+            VRgv.temp <- 0
+            if(length(temp.clus) > 1) {
+                VRgv.temp <- var(temp.rksum) * (length(temp.clus) - 1)
+            } 
+            summary.mat[which(Csize == i & Str == j), "VRgv"] <- VRgv.temp
         }
     }
         summary.mat[, "ngv"] <- summary.mat[, "Ngv"] - summary.mat[, "mgv"]
-    ## Wc is the ranksum stat
+        ## Wc is the ranksum stat
         Wc <- sum(dat[dat$grp == 1, "rksum"])
         Ngv <- summary.mat[, "Ngv"]
         mgv <- summary.mat[, "mgv"]
@@ -134,7 +138,7 @@ cluswilcox.test.ranksum.rgl.clus <- function(x, cluster, group,
         VRgv <- summary.mat[, "VRgv"]
         Rsumgv <- summary.mat[, "Rsumgv"]
         EWc <- sum(mgv * Rsumgv / Ngv)
-        VarWc <- sum(mgv * ngv / (Ngv * (Ngv - 1)) * VRgv)
+        VarWc <- sum((mgv * ngv / (Ngv * (Ngv - 1)) * VRgv)[VRgv > 0])
         Zc <- (Wc - EWc) / sqrt(VarWc)
 
         pval <- switch(alternative, less = pnorm(abs(Zc)),
@@ -158,7 +162,7 @@ cluswilcox.test.ranksum.rgl.clus <- function(x, cluster, group,
         class(result) <- "ctest"
         return(result)
     }
-
+    
 }
 
 
@@ -379,6 +383,8 @@ cluswilcox.test.ranksum.ds <- function(x, cluster, group,
     group <- group[order.c]
     cluster.uniq <- unique(cluster)
     M <- length(cluster.uniq)
+    cluster <- recoderFunc(cluster, cluster.uniq, c(1 : M))
+   
 
     ni <- table(cluster)
     
@@ -418,9 +424,9 @@ cluswilcox.test.ranksum.ds <- function(x, cluster, group,
         
         W <- numeric(M)
         for(i in 1 : M) {
-            Wi <- ((M - 1) * group[cluster == cluster.uniq[i]] -
+            Wi <- ((M - 1) * group[cluster == i] -
                    sum(ni1[-i] / ni[-i])) *
-                Fhat[cluster == cluster.uniq[i]]
+                Fhat[cluster == i]
             W[i] <- sum(Wi) / (ni[i] * (M+1))
         }
         a <- sum(ni1 / ni)
@@ -455,8 +461,8 @@ cluswilcox.test.ranksum.ds <- function(x, cluster, group,
         for( j in 1 : m) {
             for( i in 1 : M) {
                 gik.j <- ifelse(group == j, 1, 0)
-                Sj[j] <- Sj[j] + sum(gik.j[cluster == cluster.uniq[i]] *
-                                     (1 + F.prop[cluster == cluster.uniq[i]])) / ni[i]
+                Sj[j] <- Sj[j] + sum(gik.j[cluster == i] *
+                                     (1 + F.prop[cluster == i])) / ni[i]
             }
         }
         Sj <- Sj / (M + 1)
@@ -472,11 +478,11 @@ cluswilcox.test.ranksum.ds <- function(x, cluster, group,
         a <- t(d)
         for( i in 1 : M) {
             for( j in 1 : m) {
-                gik.j <- ifelse(group[cluster == cluster.uniq[i]] == j, 1, 0)
+                gik.j <- ifelse(group[cluster == i] == j, 1, 0)
                 b <- 1 / (ni[i] * (M + 1))
                 c <- gik.j * (M - 1)
                 d <- sum(a[j, -i])
-                What[j, i] <- b * sum((c - d) * Fhat[cluster == cluster.uniq[i]])
+                What[j, i] <- b * sum((c - d) * Fhat[cluster == i])
             }
         }
         EW <- matrix(0, m, M)
